package at.ac.uibk.gitsearch.repository;

import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.domain.UserWatchList;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.*;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * Spring Data  repository for the UserWatchList entity.
 */
@Repository
public interface UserWatchListRepository extends JpaRepository<UserWatchList, Long>, JpaSpecificationExecutor<UserWatchList> {

    @Query("select userWatchList from UserWatchList userWatchList where userWatchList.userId.login = ?#{principal.username}")
    List<UserWatchList> findByUserIdIsCurrentUser();
    
    @Query("select uwl from UserWatchList uwl where uwl.name LIKE CONCAT('%',:query,'%')")
    Page<UserWatchList> search(@Param("query") String query, Pageable pageable);
    
    /**
	 * Returns all watchlists of the current user.
	 *
	 * @param userId the id of the user
	 * @return the list of watchlists
	 */
	List<UserWatchList> findByUserId(long UserId);

    @Query("select uwl from UserWatchList uwl where uwl.userId.login = ?#{principal.username} AND uwl.name LIKE CONCAT('%',:query,'%')")
    Page<UserWatchList> searchForCurrentUser(@Param("query") String query, Pageable pageable);


}
